%
% batch process the NYU_CSC_TestRetest dataset (published in Shehzad et al., 2009, The Resting Brain: Unconstrained yet Reliable. Cerebral Cortex. doi:10.1093/cercor/bhn256)
%
% Steps:
% 1. Download the dataset from: http://www.nitrc.org/projects/nyu_trt/ (6 NYU_TRT_session*.tar.gz files)
% 2. Run this script. The script will:
%       a) Decompress the NYU_TRT_session*.tar.gz files into NYU_TRT_session* folders
%          Decompress the *.nii.gz files
%       b) Spatial preprocessing of the anatomical and functional volumes
%         (normalization & segmentation of anatomical volumes; realignment,
%         coregistration, normalization, and smooting of the functional
%         volumes)
%       c) Estimate first-level seed-to-voxel connectivity maps for each of 
%         the default seeds (located in the conn/rois folder), separately 
%         for each subject and for of the three test-retest sessions.
%         Estimate second-level seed-to-voxel results for each of the default
%         seeds, collapsed across the three sessions.
%
%       
%


%% UNTAR .tar.gz files
a=dir('NYU_TRT_session*.tar.gz');
for n1=1:length(a),
    [a_path,a_name,a_ext]=fileparts(a(n1).name);[nill,a_name2,a_ext2]=fileparts(a_name);
    dirname=fullfile(a_path,a_name2);
    if ~isdir(dirname),
        disp(['extracting contents from file ',a(n1).name]);
        untar(a(n1).name,dirname);
    end
end    
%% UNZIP .nii.gz files
a=strvcat(conn_dir('lfo.nii.gz'),conn_dir('mprage_skullstripped.nii.gz'));
for n1=1:size(a,1),
    [a_path,a_name,a_ext]=fileparts(a(n1,:));
    if isempty(dir(fullfile(a_path,a_name))),
        disp(['unzipping file ',a(n1,:)]);
        gunzip(deblank(a(n1,:)));
    end
end

%% FIND functional/structural files
cwd=pwd;
FUNCTIONAL_FILE=cellstr(conn_dir('lfo.nii'));
STRUCTURAL_FILE=cellstr(conn_dir('mprage_skullstripped.nii'));
nsubjects=25;
if rem(length(FUNCTIONAL_FILE),nsubjects),error('mismatch number of functional files');end
if rem(length(STRUCTURAL_FILE),nsubjects),error('mismatch number of anatomical files');end
nsessions=length(FUNCTIONAL_FILE)/nsubjects;
FUNCTIONAL_FILE=reshape(FUNCTIONAL_FILE,[nsubjects,nsessions]);
STRUCTURAL_FILE={STRUCTURAL_FILE{1:nsubjects}};
disp([num2str(size(FUNCTIONAL_FILE,1)),' subjects']);
disp([num2str(size(FUNCTIONAL_FILE,2)),' sessions']);
TR=2; % Repetition time = 2 seconds

%% PREPARES connectivity analyses (using default values for all parameters, see help conn_batch to define non-default values)
clear batch;
% CONN New experiment (realignment/coregistration/segmentation/normalization/smoothing)
batch.filename=fullfile(cwd,'conn_NYU.mat');            % New conn_*.mat experiment name
batch.New.FWHM=8;                                       % 8mm FWHM smoothing
batch.New.VOX=2;                                        % 2mm voxel size for analyses
batch.New.functionals=repmat({{}},[nsubjects,1]);       % Point to functional volumes for each subject/session
for nsub=1:nsubjects,for nses=1:nsessions,batch.New.functionals{nsub}{nses}{1}=FUNCTIONAL_FILE{nsub,nses};end; end %note: each subject's data is defined by three sessions and one single (4d) file per session
batch.New.structurals=STRUCTURAL_FILE;                  % Point to anatomical volumes for each subject
% CONN Setup                                            % Default options (uses all ROIs in conn/rois/ directory); see conn_batch for additional options 
batch.Setup.RT=TR;                                      % TR (seconds)
nconditions=nsessions;                                  % treats each session as a different condition (comment the following three lines and lines 84-86 below if you do not wish to analyze between-session differences)
batch.Setup.conditions.names=cellstr([repmat('Session',[nconditions,1]),num2str((1:nconditions)')]);
for ncond=1:nconditions,for nsub=1:nsubjects,for nses=1:nsessions,  batch.Setup.conditions.onsets{ncond}{nsub}{nses}=[];batch.Setup.conditions.durations{ncond}{nsub}{nses}=[]; end;end;end
for ncond=1:nconditions,for nsub=1:nsubjects,for nses=ncond,        batch.Setup.conditions.onsets{ncond}{nsub}{nses}=0; batch.Setup.conditions.durations{ncond}{nsub}{nses}=inf;end;end;end
batch.Setup.overwrite='Yes';                            
batch.Setup.done=1;
% CONN Preprocessing                                    % Default options (uses White Matter+CSF+realignment+conditions as confound regressors); see conn_batch for additional options 
batch.Preprocessing.filter=[0.01, 0.1];                 % frequency filter (band-pass values, in Hz)
batch.Preprocessing.done=1;
batch.Preprocessing.overwrite='Yes';
% CONN Analysis                                         % Default options (uses all ROIs in conn/rois/ as connectivity sources); see conn_batch for additional options 
batch.Analysis.done=1;
batch.Analysis.overwrite='Yes';
% CONN Results                                          % Default options (compute second-level results for all sources); see conn_batch for additional options 
batch.Results.between_subjects.effect_names={'All'};    % Second-level analyses for all subjects
batch.Results.between_subjects.contrast=[1];
batch.Results.between_conditions.effect_names=...       % All sessions, average effect across sessions
    cellstr([repmat('Session',[nconditions,1]),num2str((1:nconditions)')]);
batch.Results.between_conditions.contrast=ones(1,nconditions)/nconditions;
batch.Results.done=1;
batch.Results.overwrite='Yes';

%% RUNS analyses
conn_batch(batch);

%% CONN Display
% launches conn gui to explore results
conn
conn('load',fullfile(cwd,'conn_NYU.mat'));
conn gui_results
% Use conn_display to display any of the already computed whole-brain second-level results

